# -*- coding: utf-8 -*-
#    Asymmetric Base Framework :: Enum
#    Copyright (C) 2013, 2014  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from __future__ import absolute_import, division, print_function, unicode_literals
import six

try:
	from collections import OrderedDict
except ImportError:
	from ordereddict import OrderedDict

try:
	from itertools import zip_longest
except ImportError:
	from itertools import izip_longest as zip_longest

"""
Inspired by flufl.enum
"""

class EnumItem(object):
	def __init__(self, **attrs):
		self.__dict__.update(**attrs)
		
	def __str__(self):
		return self.label
	
	def __int__(self):
		return self.value
	
	__long__ = __int__
	
	def __eq__(self, other):
		if isinstance(other, type(self)):
			return self.value == other.value
		# XXX: Must return False and not raise an error otherwise it
		#      interferes with EnumField
		return False
	
	def __hash__(self):
		return hash((type(self), self.label, self.value))
	
	def __lt__(self, other):
		if isinstance(other, type(self)):
			return self.value < other.value
		raise TypeError("Cannot compare different Enums")
	
	__gt__ = lambda self, other: not (self < other or self == other)
	__le__ = lambda self, other: self < other or self == other
	__ge__ = lambda self, other: not self < other

class EnumMeta(type):
	
	def __new__(cls, name, bases, attributes):
		
		Choices = OrderedDict()
		new_attributes = {
			'reverse' : OrderedDict(),
			'__repr__' : lambda self: name,
		}
		items = {}
		
		meta = attributes.pop('Meta', {})
		properties = getattr(meta, 'properties', ('value', 'label'))
		
		if 'value' not in properties:
			properties = ('value',) + properties
		
		item_attrs = {
			'__repr__' : lambda self: '{0}.{1}'.format(name, self._enum_name_),
			'__reduce__' : lambda self: (self._enum_type_, (self.value,))
		}
		
		for k, v in attributes.items():
			# Enum values must be upper case
			if not k.isupper():
				# All instance methods get placed onto the enum values
				if callable(v) and getattr(v, 'im_self', None) is None and k != '__new__':
					item_attrs[k] = v
				else:
					new_attributes[k] = v
				continue
				
			item_type_attrs = {
				'label' : k.replace('_', ' ').title(),
				'display_order' : None,
				'_enum_name_' : k,
				'_enum_type_' : None
			}
			
			if not isinstance(v, tuple):
				v = (v,)
			
			if len(v) > len(properties):
				raise ValueError('Too many items for the properties')
			
			attr_updates = dict(zip_longest(properties, v))
			if attr_updates.get('label', -1) is None:
				attr_updates.pop('label')
			
			item_type_attrs.update(attr_updates)
			
			if item_type_attrs['value'] is None:
				raise ValueError('Enum values cannot be None')
			
			if not isinstance(item_type_attrs['value'], six.integer_types):
				raise TypeError('Enum ordinals must be integers')
			
			if item_type_attrs['display_order'] is None:
				item_type_attrs['display_order'] = item_type_attrs['value']
			
			items[k] = item_type_attrs
		
		ItemType = type(str('{0}Item').format(name), (EnumItem,), item_attrs)
		
		for k, v in items.items():
			new_attributes[k] = ItemType(**v)
		
		sorted_items = sorted(items.values(), key = lambda x: x['display_order'])
		
		if len(items):
		
			for index, item in enumerate(sorted_items):
				new_attributes[item['_enum_name_']].display_order = item['display_order'] = index + 1
				Choices[item['value']] = item['label']
				new_attributes['reverse'][item['value']] = new_attributes[item['_enum_name_']]
			
		new_attributes['Choices'] = Choices
		EnumType = super(EnumMeta, cls).__new__(cls, name, bases, new_attributes)
		
		for item in EnumType:
			setattr(item, '_enum_type_', EnumType)
		
		return EnumType
	
	def __iter__(self):
		return iter(self.reverse.values())

	def __contains__(self, item):
		if not isinstance(item, EnumItem):
			return False
		if item._enum_type_ != self:
			return False
		return item.value in self.Choices

class Enum(six.with_metaclass(EnumMeta)):
	'''Baseclass for Enums. 
	   ** DO NOT define methods in here'''
	Choices = {}
	
	def __new__(cls, value):
		try:
			return cls.reverse[value]
		except KeyError:
			raise ValueError('{0} has no "{1}" value'.format(cls.__name__, value))

if __name__ == '__main__':
	class MyEnum(Enum):
		A = 1
		B = 2
	
	class MyEnum2(Enum):
		A = 1
		B = 2
	
	a1 = MyEnum.A
	b1 = MyEnum.B
	a2 = MyEnum2.A
	b2 = MyEnum2.B
	
	assert MyEnum.A._enum_type_ == MyEnum
	
	assert a1 == MyEnum.A
	assert a1 != MyEnum.B
	assert a1 != MyEnum2.A
	assert a1 != a2
	assert a1 is MyEnum.A
	assert isinstance(a1, EnumItem)
	assert not isinstance(a1, type(a2))
	assert isinstance(a1, type(a2).__bases__[0])
	
	assert MyEnum.A.display_order == 1
	
	class MyEnum3(Enum):
		class Meta(object):
			properties = ('label', 'value')
		
		A = 'Hello', 3
		B = 'Foo', 1
		
		def enum(self):
			return self.label
		
		@classmethod
		def getBLabel(cls):
			return cls.B.label
	
	assert str(MyEnum3.A) == 'Hello'
	assert str(MyEnum3.B) == 'Foo'
	assert MyEnum3.A != 3
	assert MyEnum3.A.enum() == 'Hello'
	assert MyEnum3.getBLabel() == 'Foo'
	try:
		MyEnum3.enum()
		assert False, 'MyEnum3.enum() exists'
	except AttributeError:
		pass
	
	assert MyEnum3.A > MyEnum3.B
	
	try:
		_ = MyEnum3.A > MyEnum.A
		assert False, 'Should not be able to compare MyEnum3.A > MyEnum.A'
	except:
		pass
	
	assert repr(MyEnum3.A) == 'MyEnum3.A'
	
	class MyEnum4(Enum):
		A = 1
		B = 3, 'Hello'
		UPPER_CASE_WORD = 4
	
	assert str(MyEnum4.A) == 'A'
	assert str(MyEnum4.B) == 'Hello'
	assert str(MyEnum4.UPPER_CASE_WORD) == 'Upper Case Word'
	
	assert MyEnum4(1) == MyEnum4.A
	assert MyEnum4(1) is MyEnum4.A
	
	try:
		MyEnum4(2)
		assert False, 'MyEnum4(2) Should fail.'
	except ValueError:
		pass
	
	assert MyEnum4(1).label == 'A'
	
	assert tuple(MyEnum4) == (MyEnum4.A, MyEnum4.B, MyEnum4.UPPER_CASE_WORD)
	
	assert MyEnum4.A in MyEnum4
	assert MyEnum3.A not in MyEnum4
	assert 'A' not in MyEnum4
	
	assert 1 not in MyEnum4
	if 1 in MyEnum4:
		assert False, 'Cannot test ints in Enums'
	
	assert MyEnum4.A in set([MyEnum4.A, MyEnum4.B]), "Items should be hashable in sets"
	
	try:
		class MyEnum5(Enum):
			A = 'a'
			B = 'b'
		assert False, 'Enum ordinals must be integers'
	except TypeError:
		pass
	
	class MyEnum6(Enum):
		A = 1, 2
		B = 2, 3
		C = 3, 1
		
		class Meta(object):
			properties = ('value', 'display_order')
	
	assert MyEnum6.A.display_order == 2
	assert tuple(MyEnum6) == (MyEnum6.C, MyEnum6.A, MyEnum6.B)
	
	# Enums should be hashable
	hash(MyEnum6.A)
	d = {MyEnum6.A : 'a'}
	assert d[MyEnum6.A] == 'a'
	
	import pickle
	
	d = pickle.dumps(MyEnum6.A, pickle.HIGHEST_PROTOCOL)
	r = pickle.loads(d)
	
	assert r == MyEnum6.A




from __future__ import absolute_import, division, print_function, unicode_literals

from django.db import models

from asymm_enum.enum import Enum
from asymm_enum.fields.enumfield import EnumField

__author__ = "Richard Eames <reames@asymmetricventures.com>"
__date__ = "May 20, 2014"
__updated__ = "May 20, 2014"
__rev__ = "$Id$"

class TestEnum(Enum):
	VALUE1 = 1
	VALUE2 = 2

class TestEnumModel(models.Model):
	field1 = EnumField(TestEnum)

class TestEnumModel1(models.Model):
	field1 = EnumField(TestEnum)
	field2 = EnumField(TestEnum)
	field3 = EnumField(TestEnum)

class TestEnumModelWithDefault(models.Model):
	field1 = EnumField(TestEnum, default = TestEnum.VALUE1)



# -*- coding: utf-8 -*-
#    Asymmetric Base Framework :: Enum
#    Copyright (C) 2013  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import absolute_import, division, print_function, unicode_literals

import django
from django.conf import settings
from django.core.management import call_command
from django.core import exceptions
from django.db import models
from django.db.models import loading
from django.utils import unittest

import asymm_enum #@UnusedImport #This is just so that it's in `globals()`
import six

from ..fields.enumfield import EnumField
from .testapp.models import TestEnumModel, TestEnum, TestEnumModelWithDefault

if django.get_version() >= '1.7':
	from django.db import migrations  # NOQA @UnresolvedImport
	from django.db.migrations.writer import MigrationWriter  # NOQA @UnresolvedImport

class TestEnumField(unittest.TestCase):

	def setUp(self):
		loading.cache.loaded = False
		migrate = 'south' not in settings.INSTALLED_APPS
		call_command('syncdb', verbosity = 0, migrate = migrate)
		
		TestEnumModel.objects.all().delete()
		TestEnumModel.objects.bulk_create((
			TestEnumModel(field1 = TestEnum.VALUE1),
			TestEnumModel(field1 = TestEnum.VALUE1),
			TestEnumModel(field1 = TestEnum.VALUE1),
			TestEnumModel(field1 = TestEnum.VALUE2),
			TestEnumModel(field1 = TestEnum.VALUE2),
		))
		
	def test_querying1(self):
		''' Querying by Enum value '''
		self.assertEqual(TestEnumModel.objects.filter(field1 = TestEnum.VALUE1).count(), 3)
		self.assertEqual(TestEnumModel.objects.filter(field1 = TestEnum.VALUE2).count(), 2)
	
	def test_querying2(self):
		''' - Test querying by string value
		    - Test that model fields gets converted to enum
		'''
		model1 = TestEnumModel.objects.filter(field1 = '1')
		
		self.assertEqual(model1[0].field1, TestEnum.VALUE1)
		self.assertNotEqual(model1[0].field1, 1)
		
		
	def test_get_default(self):
		model = TestEnumModelWithDefault()
		model.save()
		
		self.assertEqual(model.field1, TestEnum.VALUE1)
		
	
	@unittest.skipIf(django.get_version() < '1.7', "Migrations only in django > 1.7")
	def test_migration(self):
		fields = {
			'field1':EnumField(TestEnum)
		}
		migration = type(str("Migration"), (migrations.Migration,), {
			'operations' : [
				migrations.CreateModel("Model1", tuple(fields.items()), {}, (models.Model))
			]
		})
		writer = MigrationWriter(migration)
		output = writer.as_string()
		self.assertIsInstance(output, six.binary_type)
		r = {}
		try:
			exec(output, globals(), r)
		except Exception as e:
			self.fail("Could not exec {!r}: {}".format(output.strip(), e))
		self.assertIn("Migration", r)
	
	
	def test_modelfield_validate(self):
		obj = TestEnumModel()
		field = TestEnumModel._meta.get_field_by_name('field1')[0]
		
		field.validate(TestEnum.VALUE1, obj)
		with self.assertRaises(exceptions.ValidationError):
			field.validate(0, obj)


if __name__ == "__main__":
	#import sys;sys.argv = ['', 'Test.testName']
	unittest.main()



# -*- coding: utf-8 -*-
#    Asymmetric Base Framework :: Enum
#    Copyright (C) 2013  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from __future__ import absolute_import, division, print_function, unicode_literals

from django.utils import unittest

from asymm_enum.enum import Enum

try:
	class EnumP(Enum):
		A = 1
		B = 2
except Exception as e:
	raise Exception("Could not create an enum")

class EnumTest(unittest.TestCase):
	def test_basic_enum(self):
		class Enum1(Enum):
			A = 1
			B = 2
		
		self.assertEqual(int(Enum1.A), 1)
		self.assertEqual(str(Enum1.A), 'A')
		self.assertEqual(int(Enum1.B), 2)
		self.assertEqual(str(Enum1.B), 'B')
		
		self.assertNotEqual(Enum1.A, Enum1.B)
		
		self.assertIs(Enum1.A, Enum1.A)
		
		self.assertEqual(Enum1.A._enum_type_, Enum1)
		
		self.assertEqual(Enum1(1), Enum1.A)
		self.assertIs(Enum1(1), Enum1.A)
		
		with self.assertRaises(ValueError):
			_ = Enum1(3) 
		
		with self.assertRaises(TypeError):
			# Enum ordinals must be integers
			class Enum5(Enum):
				A = 'a'
				B = 'b'
		
	def test_two_enums(self):
		class Enum1(Enum):
			A = 1
			B = 2
		
		class Enum2(Enum):
			A = 1
			B = 2
		
		self.assertNotEqual(Enum1.A, Enum2.A)
		self.assertEqual(Enum1.A, Enum1.A)
		self.assertNotEqual(Enum1.B, Enum2.B)
	
	def test_methods(self):
		class E(Enum):
			A = 1
			B = 2
		
			def mylabel(self):
				return self.label
			
			@classmethod
			def getBLabel(cls):
				return cls.B.label
		
		self.assertEqual(E.A.mylabel(), 'A')
		self.assertEqual(E.getBLabel(), 'B')
		
		self.assertRaises(AttributeError, lambda: E.A.getBLabel())
		self.assertRaises(AttributeError, lambda: E.mylabel())
	
	def test_comparitor(self):
		class Enum1(Enum):
			A = 1
			B = 2
		
		class Enum2(Enum):
			A = 1
			B = 2
		
		self.assertGreater(Enum1.B, Enum1.A)
		
		self.assertRaises(TypeError, lambda: Enum1.A > Enum2.A)
	
	def test_repr_labels(self):
		class MyEnum4(Enum):
			A = 1
			B = 3, 'Hello'
			UPPER_CASE_WORD = 4
		
		self.assertEqual(repr(MyEnum4.A), 'MyEnum4.A')
		
		self.assertEqual(str(MyEnum4.A), 'A')
		self.assertEqual(str(MyEnum4.B), 'Hello')
		self.assertEqual(str(MyEnum4.UPPER_CASE_WORD), 'Upper Case Word')
		
		self.assertEqual(MyEnum4(3).label, 'Hello')
	
	def test_membership(self):
		class MyEnum4(Enum):
			A = 1
			B = 3, 'Hello'
			UPPER_CASE_WORD = 4
		
		self.assertListEqual(list(MyEnum4), [MyEnum4.A, MyEnum4.B, MyEnum4.UPPER_CASE_WORD])
		self.assertIn(MyEnum4.A, MyEnum4)
		self.assertNotIn(1, MyEnum4)
	
	def test_display_order(self):
		class MyEnum6(Enum):
			class Meta(object):
				properties = ('value', 'display_order')
			A = 1, 2
			B = 2, 3
			C = 3, 1
		
		self.assertEqual(MyEnum6.A.display_order, 2)
		self.assertListEqual(list(MyEnum6), [MyEnum6.C, MyEnum6.A, MyEnum6.B])
	
	def test_hashable(self):
		class EnumH(Enum):
			A = 1
			B = 2
		
		_ = hash(EnumH.A)
		d = { EnumH.A : 'a'}
		self.assertEqual(d[EnumH.A], 'a')
	
	def test_pickle(self):
		import pickle
		
		d = pickle.dumps(EnumP.A, pickle.HIGHEST_PROTOCOL)
		r = pickle.loads(d)
		
		self.assertEqual(r, EnumP.A)



# -*- coding: utf-8 -*-
#    Asymmetric Base Framework - A collection of utilities for django frameworks
#    Copyright (C) 2013  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from __future__ import absolute_import, division, print_function, unicode_literals

from django import forms
from django.forms.formsets import formset_factory
from django.forms.models import modelformset_factory

from asymm_enum.fields.enumfield import EnumField

from .models import TestEnum, TestEnumModel, TestEnumModel1, TestEnumModelWithDefault

__author__ = "Richard Eames <reames@asymmetricventures.com>"
__date__ = "May 20, 2014"
__updated__ = "May 20, 2014"
__rev__ = "$Id$"

class TestForm(forms.Form):
	field1 = EnumField(TestEnum).formfield(required = False)
	field2 = EnumField(TestEnum, default = TestEnum.VALUE2).formfield()

class TestModelForm(forms.ModelForm):
	class Meta(object):
		model = TestEnumModel
		fields = ('field1',)

class TestModelWithDefaultForm(forms.ModelForm):
	class Meta(object):
		model = TestEnumModelWithDefault
		fields = ('field1',)

TestModelFormSet = modelformset_factory(TestEnumModel1, fields=('field1', 'field2', 'field2'))
TestFormSet = formset_factory(TestForm)



# -*- coding: utf-8 -*-
#    Asymmetric Base Framework :: Enum
#    Copyright (C) 2013  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from datetime import datetime
from setuptools import setup, find_packages

classifiers = """
Development Status :: 4 - Beta
Framework :: Django
Programming Language :: Python
Intended Audience :: Developers
Natural Language :: English
Operating System :: OS Independent
Topic :: Software Development :: Libraries
Topic :: Utilities
License :: OSI Approved :: GNU General Public License v2 (GPLv2)
Topic :: Software Development :: Libraries :: Application Frameworks
"""

version = '0.3.5'
url = 'https://github.com/AsymmetricVentures/asym-enum'

setup(
	name = 'asymm-enum',
	version = '{0}-{1}'.format(version, datetime.now().strftime('%Y%m%d%H%M')),
	url = url,
	download_url = '{0}/archive/v{1}.tar.gz'.format(url, version),
	author = 'Richard Eames',
	author_email = 'reames@asymmetricventures.com',
	packages = find_packages(),
	classifiers = list(filter(None, classifiers.split('\n'))),
	
	license = 'GPLv2',
	description = 'Java style enums for Python',
	
	install_requires = (
		'six',
	),
	
	#tests_require = ['django>=1.4.5'], # Travis takes care of this
	test_suite = 'run_tests.main',
)



# -*- coding: utf-8 -*-
#    Asymmetric Base Framework :: Enum
#    Copyright (C) 2013  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from __future__ import absolute_import, division, print_function, unicode_literals

from django import forms
from django.core import exceptions
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.fields import NOT_PROVIDED
from django.db.models.fields.subclassing import SubfieldBase

import six

from asymm_enum.enum import Enum, EnumItem

def _enum_coerce(self, enum, value):
	if value is None or value == '':
		return None
	
	elif isinstance(value, EnumItem) and value in enum:
		return value
	
	try:
		return enum(int(value))
	except ValueError:
		raise exceptions.ValidationError(
			self.error_messages['invalid_choice'],
			code = 'invalid_choice',
			params = {'value' : value}
		)

class EnumFormField(forms.TypedChoiceField):
	
	def __init__(self, enum, *args, **kwargs):
		self.enum = enum
		if 'choices' not in kwargs:
			kwargs['choices'] = enum.Choices.items()
		super(EnumFormField, self).__init__(*args, **kwargs)
	
	def prepare_value(self, data):
		if isinstance(data, EnumItem):
			return str(data.value)
		
		return data
	
	def valid_value(self, value):
		for k, _ in self._get_flat_choices():
			if not k and not value:
				return True
			
			if isinstance(value, EnumItem):
				if value.value == k:
					return True
		return False
	
	def to_python(self, value):
		return _enum_coerce(self, self.enum, value)
	
	def _get_flat_choices(self):
		for k, v in self.choices:
			if isinstance(v, (list, tuple)):
				for k2, v2 in v:
					yield k2, v2
			else:
				yield k, v
	
	def _has_changed(self, initial, data):
		initial_value = initial if initial is not None else ''
		try:
			data = self.to_python(data)
		except ValidationError:
			return True
		data_value = str(data.value) if data is not None else ''
		
		return initial_value != data_value
		
class EnumField(six.with_metaclass(SubfieldBase, models.IntegerField)):
	
	empty_strings_allowed = False
	validators = []
	
	def __init__(self, enum, *args, **kwargs):
		self.enum = enum
		kwargs.update(
			choices = enum.Choices.items(),
			null = False,
			blank = False
		)
		
		super(EnumField, self).__init__(*args, **kwargs)
	
	def __copy__(self, *args, **kwargs):
		return self
	
	def __deepcopy__(self, *args, **kwargs):
		return self
	
	def get_default(self):
		"""
		Returns the default value for this field.
		
		The default implementation on models.Field calls force_unicode
		on the default, which means you can't set arbitrary Python
		objects as the default. To fix this, we just return the value
		without calling force_unicode on it. Note that if you set a
		callable as a default, the field will still call it.
		
		Modified from: http://djangosnippets.org/snippets/1694/
		
		"""
		if self.has_default():
			default = self.default
			if callable(default):
				default = self.default()
			if default is None:
				return None
			if isinstance(default, six.integer_types):
				return default
			if isinstance(default, six.string_types):
				return default
			return str(default.value)
		# If the field doesn't have a default, then we punt to models.Field.
		return super(EnumField, self).get_default()
	
	def to_python(self, value):
		return _enum_coerce(self, self.enum, value)
	
	def validate(self, value, model_instance):
		if value not in self.enum:
			raise exceptions.ValidationError(
				self.error_messages['invalid_choice'],
				code='invalid_choice',
				params={'value': value},
			)
	
	def get_prep_value(self, value):
		if value in (None, ''):
			return None
		
		return int(value)
	
	def formfield(self, **kwargs):
		include_blank = (self.blank or not (self.has_default() or 'initial' in kwargs))
		defaults = {
			'enum' : self.enum,
			'required' : not self.blank,
			'label' : self.verbose_name,
			'help_text' : self.help_text,
			'choices': self.get_choices(include_blank = include_blank),
			'coerce': self.to_python,
			'empty_value' : None,
		}
		if self.has_default():
			if callable(self.default):
				defaults['initial'] = self.default
				defaults['show_hidden_initial'] = True
				
			else:
				defaults['initial'] = self.get_default()
		
		for k in kwargs.keys():
			if k not in ('coerce', 'empty_value', 'choices', 'required',
						'widget', 'label', 'initial', 'help_text',
						'error_messages', 'show_hidden_initial'):
				del kwargs[k]
		
		defaults.update(**kwargs)
		
		return EnumFormField(**defaults)
	
	def deconstruct(self):
		name, path, args, kwargs = super(EnumField, self).deconstruct()
		kwargs['enum'] = self.enum
		kwargs['default'] = self.get_default()
		
		return name, path, args, kwargs

try:
	from south.modelsinspector import add_introspection_rules
	
	def enum_converter(value):
		if issubclass(value, Enum):
			return 'Migration().gf("{0}.{1}")'.format(value.__module__, value.__name__)
		
		raise ValueError("Unknown value type `{0!r}` for enum argument".format(value))
	
	def default_converter(value):
		if isinstance(value, NOT_PROVIDED) or value is NOT_PROVIDED:
			return None
		
		elif isinstance(value, EnumItem):
			return repr(value.value)
		
		raise ValueError(repr(value))

	
	add_introspection_rules(
		[
			(
				[EnumField], # Field Name
				[], # Args
				{ # kwargs
					'enum' : ['enum', {'converter' : enum_converter, 'is_django_function' : True}],
					'default' : ['default', {'converter' : default_converter, 'is_django_function' : True}],
				}
			)
		],
		['^asymm_enum\.fields\.enumfield\.EnumField']
	)
except ImportError:
	pass



# -*- coding: utf-8 -*-
#    Asymmetric Base Framework - A collection of utilities for django frameworks
#    Copyright (C) 2013  Asymmetric Ventures Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import absolute_import, division, print_function, unicode_literals

from django.test.client import RequestFactory
from django.test.testcases import SimpleTestCase, assert_and_parse_html
from django.utils import unittest

from .testapp.forms import TestModelForm, TestModelWithDefaultForm, TestFormSet
from .testapp.models import TestEnum, TestEnumModel

class FormTest(SimpleTestCase):
	
	def setUp(self):
		self.factory = RequestFactory()
	
	# Borrowed from django for compatibility with django 1.4
	def assertInHTML(self, needle, haystack, count = None, msg_prefix = ''):
		needle = assert_and_parse_html(self, needle, None,
			'First argument is not valid HTML:')
		haystack = assert_and_parse_html(self, haystack, None,
			'Second argument is not valid HTML:')
		real_count = haystack.count(needle)
		if count is not None:
			self.assertEqual(real_count, count,
				msg_prefix + "Found %d instances of '%s' in response"
				" (expected %d)" % (real_count, needle, count))
		else:
			self.assertTrue(real_count != 0,
				msg_prefix + "Couldn't find '%s' in response" % needle)
	
	def test_validation1(self):
		''''''
		request = self.factory.post('/', {'field1' : '1'})
		
		form = TestModelForm(request.POST)
		self.assertTrue(form.is_valid())
		
		models = form.save()
		self.assertEqual(models.field1, TestEnum.VALUE1)
	
	def test_validation2(self):
		''' Test validation against invalid value'''
		request = self.factory.post('/', {'field1' : '8'})
		form = TestModelForm(request.POST)
		
		self.assertFalse(form.is_valid())
	
	def test_widget_selected(self):
		model = TestEnumModel()
		model.field1 = TestEnum.VALUE2
		
		form = TestModelForm(instance = model)
		
		substring_html = '<option value="{0}" selected="selected">{1}</option>'.format(
			int(TestEnum.VALUE2), str(TestEnum.VALUE2)
		)
		
		self.assertInHTML(substring_html, str(form))
	
	def test_widget_with_initial(self):
		form = TestModelWithDefaultForm()
		
		substring_html = '<option value="{0}" selected="selected">{1}</option>'.format(
			int(TestEnum.VALUE1), str(TestEnum.VALUE1)
		)
		
		self.assertInHTML(substring_html, str(form))
	
	def test_with_formset1(self):
		request = self.factory.post('/', {
			'form-TOTAL_FORMS' : '1',
			'form-INITIAL_FORMS' : '0',
			'form-MAX_NUM_FORMS' : '0',
			'form-0-field1' : '',
			'form-0-field2' : '2',
		})
		
		formset = TestFormSet(request.POST)
		
		self.assertFalse(formset[0].has_changed())
		self.assertFalse(formset.has_changed())
		self.assertTrue(formset.is_valid())
		

if __name__ == "__main__":
	#import sys;sys.argv = ['', 'Test.testName']
	unittest.main()
